--[[

	File: maxpassive.lua
	Author: Morten Larsen, Hex Script Project, http://hexscript.strangled.net
	Enviroment: HeXHub 5.01j/HexScript 1.0.1.0 or newer
	Description: This script limits the amount of passive user allowed into the hub.
	             The script does not check users with "info1" right (the right to see/use OpChat)
	             If a active mode users changes from to passive, that user is now counted as a passive user (works in reverse too)
]]

-- Settings

-- Max number of passive clients allowed to login to the hub
iMaxPassive = 25

-- The address where passive users are redirected to if iMaxPassive is exceeded
sRedirect = "nemesis.te-home.net"

-- A message sent to passive users that were allowed to login
sPassiveMessage = "Your client is configured for passive mode! Please configure your client for active mode, it will offer you more search results and sources for downloads. For more information about how to configure your client for active mode please visit: http://www.dslreports.com/faq/6518"

-- The message sent to passive users who are about to be disconnected
sDisconnectMessage = "Too many passive mode users in this hub already, try again later... For more information about how to configure your client for active mode please visit: http://www.dslreports.com/faq/6518"

-- The script..
tCurPasUsers = {}
hubbot = Hex.User(0)
tmr = Hex.Timer()

Hex.SetAboutString("Morten Larsen, 2008")

function OnLoad()
	if tmr:Start(30000, 30000, "TimerProc") == false then
		print("maxpassive.lua: failed to start timer, script aborting...")
		
		-- stop execution of the script
		return false
	end
	
	return true
end

-- Check the M: variable in the tag in the $MyINFO's description field.
-- If not active and max passive users is exceeded the user will be disconnected
function OnCommand(iUserId, sData)
	local index = nil
	local value = nil
	
	-- check if this is a valid myinfo
	local _, _, n, d, t, c, e, s = string.find(sData, "^$MyINFO $ALL (%S+)%s+([^$]*)<([^<]*)>$ $([^$]*)$([^$]*)$([^$]+)")
	if n ~= nil then 
		if string.find(t, "M:A,") ~= nil then 
			-- Check to see if this user changed mode from passive to active
			for index, value in ipairs(tCurPasUsers) do
				if (value == iUserId) then
					table.remove(tCurPasUsers, i)
					iCurNumPasUsers = iCurNumPasUsers - 1
					return true
				end
			end -- for
		else
			-- Dont check operators (ppl that have access to opchat)
			local r = Hex.Right(iUserId)
			if r:CheckRightByName("info1") == true then return true end
			
			-- Check to see if this passive user is already counted
			for index, value in ipairs(tCurPasUsers) do
				if (value == iUserId) then
					return true
				end
			end
			
			-- this is a new user, or an active mode user changing to passive.
			local usr = Hex.User(iUserId)
			if #tCurPasUsers >= iMaxPassive then
				usr:SendData("<"..hubbot:GetNick().."> "..sDisconnectMessage.."|$ForceMove "..sRedirect.."|")
				usr:Disconnect()
				return false
			else
				table.insert(tCurPasUsers, iUserId)
				usr:SendData("<"..hubbot:GetNick().."> "..sPassiveMessage.."|")
			end
		end	-- find mode
	end -- find $myinfo
	return true
end

function OnUserJoined(iUserId)
	local r = Hex.Right(iUserId)
	if r:CheckRightByName("info1") == true then 
		-- make sure operators is not counted, they might be in OnCommand if not identified yet
		OnUserParts(iUserId)
	end
end

-- if a passive user left the hub then remove the user from our collection
function OnUserParts(iUserId)
	local index = nil
	local value = nil
	
	for index, value in ipairs(tCurPasUsers) do
		if (value == iUserId) then
			table.remove(tCurPasUsers, index)
			break
		end
	end
end

-- Just in case someone was not properly removed from the collection.
-- E.g. we add before OnUserJoined is fired, and hexhub might decide to disconnect the user before this event.
function TimerProc(iTimerId)
	for i=#tCurPasUsers,1,-1 do
		if (IsUserOnline(tCurPasUsers[i]) == false) then
			table.remove(tCurPasUsers, i)
		end
	end
end

function IsUserOnline(iUserId)
	local usr = Hex.User(-1)
	local bResult = true
	
	while bResult == true do
		if iUserId == usr:GetUserID() then return true end
		bResult = usr:Next()
	end
		
	return false
end